/*********************************************************************/
/*                                                                   */
/*  This Program Written By Paul Edwards.                            */
/*  Released to the public domain.                                   */
/*                                                                   */
/*********************************************************************/
/*********************************************************************/
/*                                                                   */
/*  diffsync - synch up a diff file with the files you really have   */
/*                                                                   */
/*  If you have a normal diff from the diff program, but your files  */
/*  are in fact changed slightly since then, perhaps something       */
/*  similar to whitespace (e.g. MVS line numbers) but not really     */
/*  whitespace, then this program will refresh the diff so that      */
/*  there are no complaints.                                         */
/*                                                                   */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXBUF 32767

static char buf[MAXBUF];

int main(int argc, char **argv)
{
    FILE *l;
    FILE *r;
    long left = 0;
    long right = 0;

    if (argc < 3)
    {
        fprintf(stderr, "usage: diffsync <left> <right>\n");
        fprintf(stderr, "reads diff file from stdin, writes to stdout\n");
        return (EXIT_FAILURE);
    }
    
    l = fopen(*(argv + 1), "r");
    if (l == NULL)
    {
        fprintf(stderr, "failed to open %s\n", *(argv + 1));
        return (EXIT_FAILURE);
    }
    
    r = fopen(*(argv + 2), "r");
    if (r == NULL)
    {
        fprintf(stderr, "failed to open %s\n", *(argv + 2));
        return (EXIT_FAILURE);
    }
    
    while (buf[1] = ' ', fgets(buf, sizeof buf, stdin) != NULL)
    {
        if (buf[0] == '<')
        {
            fgets(buf + 2, sizeof buf - 2, l);
            left++;
        }
        else if (buf[0] == '>')
        {
            fgets(buf + 2, sizeof buf - 2, r);
            right++;
        }
        /* if we have a change group, need to reposition left and
           right files */
        else if (buf[0] != '-')
        {
            char grp[100];
            char *p;
            long lsync = left;
            long rsync = right;
            
            strncpy(grp, buf, sizeof grp);
            grp[sizeof grp - 1] = '\0';
            
            if (strchr(buf, 'd') != NULL)
            {
                lsync = atoi(buf);
            }
            else if ((p = strchr(buf, 'a')) != NULL)
            {
                rsync = atoi(p + 1);
            }
            else if ((p = strchr(buf, 'c')) != NULL)
            {
                lsync = atoi(buf);
                rsync = atoi(p + 1);
            }
            
            while ((left + 1) < lsync)
            {
                fgets(buf, sizeof buf, l);
                left++;
            }
            while ((right + 1) < rsync)
            {
                fgets(buf, sizeof buf, r);
                right++;
            }
            strcpy(buf, grp);
        }
        /* If we have a blank line, don't include the space */
        if (buf[2] == '\n')
        {
            strcpy(buf + 1, "\n");
        }
        fputs(buf, stdout);
    }
    return (0);
}
