/*********************************************************************/
/*                                                                   */
/*  This Program Written by Paul Edwards.                            */
/*  Released to the Public Domain                                    */
/*                                                                   */
/*********************************************************************/
/*********************************************************************/
/*                                                                   */
/*  dssfix - fix dfdss dumps that have been FTPed in binary mode     */
/*  and have thus lost the block boundaries. This program restores   */
/*  those boundaries into a sensible format (RECFM=V) that actually  */
/*  protects such data boundaries with an RDW, which then allows     */
/*  you to do an IDCAMS REPRO from V to U and then DFDSS (ADRDSSU)   */
/*  should be able to do a restore.                                  */
/*                                                                   */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static char bigbuf[33000];
static unsigned char rdw[4];
static unsigned char header[8];

int main(int argc, char **argv)
{
    FILE *fp;
    FILE *fq;
    unsigned int len;

    if (argc != 3)
    {
        printf("usage: dssfix <in file> <out file>\n");
        printf("e.g. dssfix dd:in dd:out\n");
        printf("input should probably be RECFM=U\n");
        printf("output should probably be RECFM=V\n");
        return (EXIT_FAILURE);
    }

    fp = fopen(*(argv + 1), "rb");
    if (fp == NULL)
    {
        printf("can't open %s\n", *(argv + 1));
        return (EXIT_FAILURE);
    }

    fq = fopen(*(argv + 2), "wb");
    if (fq == NULL)
    {
        printf("can't open %s\n", *(argv + 2));
        return (EXIT_FAILURE);
    }

    while (!feof(fp) && !ferror(fp) && !ferror(fq))
    {
        fread(header, 1, sizeof header, fp);
        if (!feof(fp))
        {
            len = ((unsigned int)header[6] << 8) | header[7];
            if (len < (sizeof header))
            {
                /* assume NUL-padding at end of FB file */
                break;
            }
            if (len > (32767 - sizeof rdw))
            {
                printf("length of %u cannot be handled\n", len);
                return (EXIT_FAILURE);
            }
            len += sizeof rdw;
            rdw[0] = (unsigned char)(len >> 8);
            rdw[1] = (unsigned char)(len & 0xff);
            fwrite(rdw, 1, sizeof rdw, fq);
            len -= (sizeof rdw + sizeof header);
            fread(bigbuf, 1, len, fp);
            if (!feof(fp) && !ferror(fp))
            {
                fwrite(header, 1, sizeof header, fq);
                fwrite(bigbuf, 1, len, fq);
            }
        }
    }

    if (ferror(fp) || ferror(fq))
    {
        printf("i/o error\n");
        return (EXIT_FAILURE);
    }

    printf("processing complete\n");

    return (0);
}
