/*********************************************************************/
/*                                                                   */
/*  This Program Written by Paul Edwards.                            */
/*  Released to the Public Domain                                    */
/*                                                                   */
/*********************************************************************/
/*********************************************************************/
/*                                                                   */
/*  modhunt.c - hunt a module for hex string                         */
/*                                                                   */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#define MAXMOD 1000000
#define MALSIZE MAXMOD+16+1

static unsigned char *buf;

#define MAXSRCH 100

#define LEFT_EXCEPTION 0x100
#define RIGHT_EXCEPTION 0x200

static int hexsrch[MAXSRCH];

int main(int argc, char **argv)
{
    char modname[9];
    int ret;
    size_t x;
    size_t y;
    char *srch;
    size_t maxpos;
    size_t len;
    char *hexdigs = "0123456789ABCDEF";
    unsigned char *p;

    if (argc <= 2)
    {
        printf("usage: modhunt <module name> <hex string>\n");
        printf("e.g. modhunt GCC 12BB34XX56\n");
        return (EXIT_FAILURE);
    }
    
    buf = malloc(MALSIZE); /* extra room for lazy algorithm */
    if (buf == NULL)
    {
        printf("insufficient memory - need %d bytes\n", MALSIZE);
        return (EXIT_FAILURE);
    }
    
    srch = *(argv + 2);
    if (strlen(srch)/2 >= (MAXSRCH - 1))
    {
        printf("search string too long - max is %d\n", MAXSRCH - 1);
        return (EXIT_FAILURE);
    }

    p = (unsigned char *)srch;
    x = 0;
    while (*p != '\0')
    {
        int ch;

        while (*p == ' ') p++;
        if (*p == '\0') break;
        ch = *p;
        ch = toupper(ch);
        if (isxdigit(ch))
        {
            hexsrch[x] = strchr(hexdigs, ch) - hexdigs;
            hexsrch[x] <<= 4;
        }
        else if (ch == 'X')
        {
            hexsrch[x] = LEFT_EXCEPTION;
        }
        else
        {
            printf("malformed search string character %c\n", *p);
            return (EXIT_FAILURE);
        }
        y++;
        p++;
        while (*p == ' ') p++;
        if (*p == '\0')
        {
            hexsrch[x++] |= RIGHT_EXCEPTION;
            break;
        }
        ch = (unsigned)*p;
        ch = toupper(ch);
        if (isxdigit(ch))
        {
            hexsrch[x] |= strchr(hexdigs, ch) - hexdigs;
        }
        else if (ch == 'X')
        {
            hexsrch[x] |= RIGHT_EXCEPTION;
        }
        else
        {
            printf("malformed search string character %c\n", *p);
            return (EXIT_FAILURE);
        }
        p++;
        x++;
    }

    len = x;

    strncpy(modname, *(argv + 1), sizeof modname);
    modname[sizeof modname - 1] = '\0';
    memset(modname + strlen(modname), ' ', 
           sizeof modname - strlen(modname) - 1);
    memset(buf, '\0', MALSIZE);
    ret = __load(modname, buf);
    printf("searching module %s with entry point %x\n", modname, ret);
    maxpos = MAXMOD - len;
    for (x = 0; x < maxpos; x++)
    {
        int ch;
        
        for (y = 0; y < len; y++)
        {
            int ch;

            ch = buf[x + y];
            /* printf("comparing ch %x and hex %x\n", ch, hexsrch[y]); */
            if (ch == hexsrch[y]) continue;
            if ((hexsrch[y] & (LEFT_EXCEPTION|RIGHT_EXCEPTION)) == 0) break;
            if (((ch & 0xf0) != (hexsrch[y] & 0xf0))
                && ((hexsrch[y] & LEFT_EXCEPTION) == 0))
            {
                break;
            }
            if (((ch & 0x0f) != (hexsrch[y] & 0x0f))
                && ((hexsrch[y] & RIGHT_EXCEPTION) == 0))
            {
                break;
            }
            continue;
        }
        if (y == len)
        {
            printf("%-8s %06X ", modname, x);
            for (y = 0; y < 4; y++)
            {
                int z;

                for (z = 0; z < 4; z++)
                {
                    printf("%02X", buf[x + y * 4 + z]);
                }
                if (y != 3)
                {
                    printf(" ");
                }
            }
            printf("\n");
        }
    }
    return (0);
}
