/*********************************************************************/
/*                                                                   */
/*  This Program Written by Paul Edwards.                            */
/*  Released to the Public Domain                                    */
/*                                                                   */
/*********************************************************************/
/*********************************************************************/
/*                                                                   */
/*  striprdw - strip RDWs from a dataset                             */
/*                                                                   */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static char bigbuf[33000];
static unsigned char rdw[4];

int main(int argc, char **argv)
{
    FILE *fp;
    FILE *fq;
    unsigned int len;

    if (argc != 3)
    {
        printf("usage: striprdw <in file> <out file>\n");
        printf("e.g. striprdw dd:in dd:out\n");
        printf("input should probably be RECFM=U or V\n");
        printf("output should probably be RECFM=U\n");
        return (EXIT_FAILURE);
    }

    fp = fopen(*(argv + 1), "rb");
    if (fp == NULL)
    {
        printf("can't open %s\n", *(argv + 1));
        return (EXIT_FAILURE);
    }

    fq = fopen(*(argv + 2), "wb");
    if (fq == NULL)
    {
        printf("can't open %s\n", *(argv + 2));
        return (EXIT_FAILURE);
    }

    while (!feof(fp) && !ferror(fp) && !ferror(fq))
    {
        fread(rdw, 1, sizeof rdw, fp);
        if (!feof(fp))
        {
            len = ((unsigned int)rdw[0] << 8) | rdw[1];
            if (len <= sizeof rdw)
            {
                /* assume NUL-padding at end of FB file */
                break;
            }
            if (len > 32767)
            {
                printf("length of %u cannot be handled\n", len);
                return (EXIT_FAILURE);
            }
            len -= sizeof rdw;
            fread(bigbuf, 1, len, fp);
            if (!feof(fp) && !ferror(fp))
            {
                fwrite(bigbuf, 1, len, fq);
            }
        }
    }

    if (ferror(fp) || ferror(fq))
    {
        printf("i/o error\n");
        return (EXIT_FAILURE);
    }

    printf("processing complete\n");

    return (0);
}
